//**************************************************************************
//
//    Copyright 1999  Broadcom Corporation
//    All Rights Reserved
//    No portions of this material may be reproduced in any form without the
//    written permission of:
//             Broadcom Corporation
//             16251 Laguna Canyon Road
//             Irvine, California  92618
//    All information contained in this document is Broadcom Corporation
//    company private, proprietary, and trade secret.
//
//
//
//**************************************************************************
//    Filename: bcmos.c
//    Author:   Dannie Gay
//    Creation Date: 21-feb-99
//
//**************************************************************************
//    Description:
//
//	os specific definitions/functions
//		
//**************************************************************************
//    Revision History:
//
//		0.1  initial revision
//
//**************************************************************************

#include <vxWorks.h>
#include <sysLib.h>
#include <tickLib.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#include "bcmos.h"
#include "board.h"

/* test function declaration */
extern void bootPrint( char *pChr); /* defined in syslib.c */


// These are the pSOS thread priority values that correspond to the portable
// ThreadPriority values.
// For the CM HAL we use a default table size and an index that corresponds with
// equivalent table in application
static const unsigned long gHalPriorityConversionTable[HAL_THREAD_PRIORITY_TABLE_LENGTH] =
{
    HAL_THREAD_PRIORITY,  // default priority
                
    150,        // I could choose a lower value, but I don't think it's necessary.

    190,        // Space them apart by 5 to allow for future expansion...(it
    195,        // doesn't matter whether the spacing is 5 or 1 as far as pSOS
    200,        // is concerned).
    205,
    210,

    230         // This is the highest value that application tasks can use.
};


//***************************************************************************
//	void Sleep 
//
//	delays a thread 
//
//   Inputs:
//       number of ticks to delay
//
//	
//***************************************************************************

void Sleep(int x)
{
    taskDelay(x);
}

//***************************************************************************
//	void	WaitLoop
//
//	 waits for n milliseconds
//
//
//***************************************************************************
void WaitLoop(unsigned long msecs)
{
   Delayms((unsigned short)msecs);
}

//***************************************************************************
//	void	WaitLoopMicro
//
//	 waits for n microseconds
//
//
//***************************************************************************
void WaitLoopMicro(unsigned long usecs)
{
    Delayus((uint32)usecs);
}


//***************************************************************************
//	uint32	BcmMuCreate
//
//	 Create a mutex object
//
//   Inputs:
//       BcmHandle * mu - pointer mutex handle (returned)
//
//	returns 0 on success, > 0 on error
//
//***************************************************************************
uint32 BcmMuCreate(BcmHandle * mu)
{
	*mu = (BcmHandle)semMCreate(SEM_Q_PRIORITY|SEM_INVERSION_SAFE);
	if( *mu )		//success
		return(0);
	else			//return NULL
		return(1);

}

//***************************************************************************
//	uint32	BcmMuLock
//
//	 Locks a mutex object (Blocking)
//
//   Inputs:
//       BcmHandle mu - mutex handle
//
//	returns 0 on success, > 0 on error
//
//***************************************************************************
uint32 BcmMuLock(BcmHandle mu)
{
    return semTake((SEM_ID)mu, WAIT_FOREVER);
}

//***************************************************************************
//	uint32	BcmMuUnlock
//
//	 Unlock the mutex object
//
//   Inputs:
//       BcmHandle mu - mutex handle
//
//	returns 0 on success, > 0 on error
//
//***************************************************************************
uint32 BcmMuUnlock(BcmHandle  mu)
{
   return semGive((SEM_ID)mu);
}

//***************************************************************************
//	uint32	BcmMuDelete
//
//	 Delete the mutex object
//
//   Inputs:
//       BcmHandle mu - mutex handle
//
//	returns 0 on success, > 0 on error
//
//***************************************************************************
uint32 BcmMuDelete(BcmHandle mu)
{
    return semDelete((SEM_ID)mu);
}

//***************************************************************************
//	uint32	BcmSemCreate
//
//	 Create a semaphore object
//
//   Inputs:
//       BcmHandle * sem - pointer semaphore handle (returned)
//
//	returns 0 on success, > 0 on error
//
//***************************************************************************
uint32 BcmSemCreate(BcmHandle * sem)
{
   static char semcnt1=0x30;
   static char semcnt2=0x30;
   static char semName[4] = "smxx";

   // check if out of range
   if (semcnt2 > 0x39)
	  return -1;

   // set semaphore name
   semName[2] = semcnt2;
   semName[3] = semcnt1;

   // increment semaphore count/name
   if (semcnt1++ > 0x39) {
	  semcnt1 = 0x30;
	  semcnt2++;
   }

   return sm_create(semName, 1, SM_LOCAL|SM_FIFO, (UINT32 *)sem);
}


//***************************************************************************
//	uint32	BcmSemWait
//
//	 Lock a semaphore object (Blocking)
//
//   Inputs:
//       BcmHandle sem - semaphore handle
//
//	returns 0 on success, > 0 on error
//
//***************************************************************************
uint32 BcmSemLock(BcmHandle sem)
{
    return sm_p(sem, SM_WAIT, 0);
}

//***************************************************************************
//	uint32	BcmSemUnlock
//
//	 Unlock the semaphore object
//
//   Inputs:
//       BcmHandle sem - semaphore handle
//
//	returns 0 on success, > 0 on error
//
//***************************************************************************
uint32 BcmSemUnlock(BcmHandle  sem)
{
   return sm_v(sem);
}

//***************************************************************************
//	int32	BcmCreateEvent
//
//	 Creates an event object
//
//	returns uint32 on success, = 0 on error
//
//***************************************************************************
uint32 BcmCreateEvent(void)
{
   // initial event seed is 0
   static uint32  evSeedMask=0x00000001;
   uint32 event;

   // check event boundary conditions
   if (evSeedMask & 0x80000000)
	  return 0;

   // assign event and adjust for next one (PSOS)
   event = evSeedMask;
   evSeedMask <<= 1;

   return event;

}


//***************************************************************************
//	int32	BcmWaitForEvent
//
//	 Waits (blocks) on the indicated event object
//
//  uint32 - event - event(s) to wait on: logical OR of all event handles
//  uint32 * rEvent -  event returned
//  uint32 - msecs - number of milliseconds to wait for the event: typically
//                   set to  = 0 to wait forever (modulo 10 msecs)
//
//	returns 0 on success, > 0 on error
//
//***************************************************************************
int32 BcmWaitForEvent(uint32 event, uint32 * rEvent, uint32 msecs)
{
   uint32 ticks;

//   int sysClkRateGet (void) return ticks per second
   ticks = (msecs * sysClkRateGet()) / 1000 ;

   // wait for event (PSOS)
   return((int32)ev_receive(event, EV_WAIT|EV_ANY, ticks, (UINT32*)rEvent));

}

//***************************************************************************
//	int32	BcmSendEvent
//
//	 Sends the indicated event object
//
//	returns 0 on success, > 0 on error
//
//***************************************************************************
int32 BcmSendEvent(uint32 event, uint32 tid)
{

   // send the event, to specific task (PSOS)
   return(ev_send(tid, event));

}

//***************************************************************************
//	uint32	BcmMsgQCreate
//
//	 Creates a system message queue of size MsgQSize
//
//	returns message queue ID on success, NULL on failure.
//
//***************************************************************************
uint32 BcmMsgQCreate(char *MsgQName, uint32 MsgQSize, uint32 MaxMsgSize)
{
   MSG_Q_ID Qid;

   //Save off the max 
   
   Qid = msgQCreate( MsgQSize,      /* max messages that can be queued */
                     MaxMsgSize,    /* max bytes in a message */
                     MSG_Q_FIFO     /* message queue options */
                     );

   return (uint32)Qid;

}

//***************************************************************************
//	uint32	BcmMsgQDelete
//
//	 Deletes a system message queue.
//
//***************************************************************************
void   BcmMsgQDelete(uint32 MsgQid)
{
   msgQDelete((MSG_Q_ID)MsgQid);
   return;
}

//***************************************************************************
//	bool	BcmMsgQSend
//
//	 Posts a message to a system queue. 
//
//	returns 1 on success, 0 on error
//
//***************************************************************************
CM_STATUS   BcmMsgQSend(uint32 msgQId, ULONG *Msg, uint16 MsgLen)
{
   STATUS QSendStatus;
   
   //Put message on queue.
   QSendStatus = msgQSend(
                           (MSG_Q_ID)msgQId,    /* message queue on which to send */
                           (char *)Msg,         /* message to send */
                           MsgLen,              /* length of message */
                           NO_WAIT,             /* ticks to wait */
                           MSG_PRI_NORMAL       /* MSG_PRI_NORMAL or MSG_PRI_URGENT */
                           );

   //Turn vxWorks status into HAL status.
   if(QSendStatus==OK)
      return CM_STATUS_SUCCESS;
   else
      return STATUS_UNSUCCESSFUL;

}

//***************************************************************************
//	bool  BcmMsgQReceive
//
//	 Gets a message from a system queue.
//
//	returns 1 on success, 0 on error
//
//***************************************************************************
CM_STATUS BcmMsgQReceive(uint32 msgQId, 
                         ULONG *AvailMsgBuffer, 
                         uint32 AvailMsgBufferLen, 
                         uint32 *RcvdMsgBufferLen,
                         BOOL WaitFlag)
{
   
   //Get a pending message.
   if(WaitFlag)
      *RcvdMsgBufferLen = msgQReceive(
                                       (MSG_Q_ID)msgQId,       /* message queue from which to receive */
                                       (char *)AvailMsgBuffer, /* buffer to receive message */
                                       AvailMsgBufferLen,      /* length of buffer */
                                       WAIT_FOREVER            /* ticks to wait */
                                       );
   else
      *RcvdMsgBufferLen = msgQReceive(
                                       (MSG_Q_ID)msgQId,       /* message queue from which to receive */
                                       (char *)AvailMsgBuffer, /* buffer to receive message */
                                       AvailMsgBufferLen,      /* length of buffer */
                                       NO_WAIT                 /* ticks to wait */
                                       );

   //Turn vxWorks status into HAL status.
   if(*RcvdMsgBufferLen==0)
      return STATUS_UNSUCCESSFUL;
   else
      return CM_STATUS_SUCCESS;

}

//***************************************************************************
//	int32	BcmCreateThread
//
//	 Creates a given thread of execution
//
//	returns 0 on success, > 0 on error
//
//***************************************************************************
int32 BcmCreateThread(char * tname, uint32 tprio, uint32 tstack, uint32 * tid)
{
   // use tprio as index into gHalPriorityConversionTable
   // check boundary condition
#if 0
   if (tprio > (HAL_THREAD_PRIORITY_TABLE_LENGTH-1))
      tprio = HAL_THREAD_PRIORITY;  // default priority
   else 
      if (!(tprio=gHalPriorityConversionTable[tprio]))
         tprio = HAL_THREAD_PRIORITY;  // default priority
#endif
	// klmod: Found in vxWorks, our Console thread priority(100) cannot be the 
	//		  same as MacInterruptHandler(CMHL) thread(230). Temp workarround:
	// klmod: for tprio < (HAL_THREAD_PRIORITY_TABLE_LENGTH-1)), use the table
	// klmod: for tprio < 256, use the number directly, backward compatibility
	// klmod: for tprio >, (uninitialize value), use default
	if (tprio < HAL_THREAD_PRIORITY_TABLE_LENGTH )	//use table
	{
		if (!(tprio=gHalPriorityConversionTable[tprio]))	//if table value is zero,
		   tprio = HAL_THREAD_PRIORITY;  // default priority
	}
	else if ( tprio > 255 )		//unintialized priority
		tprio = HAL_THREAD_PRIORITY;  // default priority

	HalPrint("Create task %s, priority set to %d\n", tname, tprio );

   return(t_create(tname,
				   tprio,
				   tstack, 0, 0,
				   (UINT32*)tid));

}

//***************************************************************************
//	int32	BcmStartThread
//
//	 Starts a given thread of execution
//
//	returns 0 on success, > 0 on error
//
//***************************************************************************
int32 BcmStartThread(uint32 tid, uint32 (*tfunc)(void *), void * targs)
{

    return(t_start(tid, T_SUPV | T_PREEMPT | T_NOTSLICE | T_ISR,
//   return(t_start(tid, T_SUPV | T_PREEMPT | T_TSLICE,
				  (void (*)())tfunc,
   				  targs));

}

//***************************************************************************
//	int32	BcmSuspendThread
//
//	 Suspends a given thread of execution
//
//	returns 0 on success, > 0 on error
//
//***************************************************************************
int32 BcmSuspendThread(uint32 tid)
{

    return(t_suspend(tid));

}

//***************************************************************************
//	int32	BcmResumeThread
//
//	 Resumes a given thread of execution
//
//	returns 0 on success, > 0 on error
//
//***************************************************************************
int32 BcmResumeThread(uint32 tid)
{

    return(t_resume(tid));

}

//***************************************************************************
//	int32  BcmStartTimerPeriodic
//
//	 Starts a periodic timer, triggers event when timer completed
//
//	returns thandle on success, 0 on error
//
//***************************************************************************
uint32 BcmStartTimerPeriodic(int32 ticks, uint32 event)
{
   uint32 t_handle=0;

   if (tm_evevery((UINT32)ticks, event, (UINT32*)&t_handle))
	  return 0;

   return t_handle;

}


//***************************************************************************
//	uint32  BcmStartTimerOneShot
//
//	 Starts a one-shot timer, triggers event when timer completed
//
//	returns t_handle on success, 0 on error
//
//***************************************************************************
uint32 BcmStartTimerOneShot(int32 ticks, uint32 event)
{
   uint32 t_handle=0;

   if (tm_evafter(ticks, event, (UINT32*)&t_handle))
	  return 0;

   return t_handle;

}


//***************************************************************************
//	int32  BcmCancelTimer
//
//	 Cancels a timer
//
//	returns  0 on success, otherwise error
//
//***************************************************************************
uint32 BcmCancelTimer(uint32 * t_handle)
{
   if (*t_handle == 0)	// timer already cancelled, no handle
	  return 0;
   else if (tm_cancel(*t_handle))
	  return -1;
   else {
	  *t_handle=0; // clr timer handle
	  return 0;
   }
}

// debug zone switch
uint16 DebugZone = 0x0001; //0xffff;

#if (BCM_REDUCED_IMAGE_SIZE)
	// Compile out here...
#else
//***************************************************************************
//	int  DEBUGMSG
//
// Print a message base on zone setting.  Takes printf style argument list.
//
//	returns  length of print
//
//***************************************************************************

int DEBUGMSG(int zone, char *FmtStr, ...)
{

   va_list Args;
   static char Text[128]; // IMPORTANT: never call w/ string len >= 128!!!
   int Length;

   // check if zone is active
   if (!((uint16)zone & DebugZone))
	  return 0;

   va_start (Args, FmtStr);
   Length = vsprintf (Text, FmtStr, Args);
   va_end (Args);

#if ( TORNADO_DEBUGGING==1 )
    printf(Text);
#else

#ifdef INCLUDE_DDB5476
   printf(Text);
#else
   bootPrint(Text);
#endif

#endif /* #if ( TORNADO_DEBUGGING==1 ) */

   return Length;

}

//***************************************************************************
//	void  LogRawBuffer
//
// Debug Hex dump
//
//	returns  length of print
//
//***************************************************************************
void LogRawBuffer (char * str, UCHAR *pBuffer, int BufLen, BOOL wacky)
{
   int Loop;
   char DataLine[96];
   char DataValue[16];

   printf("%s\nLogging %d bytes:\r\n", str, BufLen);

   DataLine[0] = 0;
   for (Loop = 0; Loop < BufLen; Loop++) {
	  sprintf (DataValue, "%02x ", pBuffer[Loop]);
	  strcat (DataLine, DataValue);

	  if ((Loop + 1) % 16 == 0) {
		 strcat (DataLine, "\r\n");
		 printf("%s\n", DataLine);
		 DataLine[0] = 0;
	  }

	  else if ((Loop + 1) % 4 == 0)
		 strcat (DataLine, "  ");
   }

   strcat (DataLine, "\r\n");
   printf("%s\n", DataLine);
}

#endif // (BCM_REDUCED_IMAGE_SIZE)



//***************************************************************************
//	int  Print
//
// Print a message regardless. Direct access into the serial port registers
//
//	returns  length of print
//
//***************************************************************************
int Print(char *FmtStr, ...)
{

   va_list Args;
   static char Text[128]; // IMPORTANT: never call w/ string len >= 128!!!
   int Length;

   va_start (Args, FmtStr);
   Length = vsprintf (Text, FmtStr, Args);
   va_end (Args);
#ifdef INCLUDE_DDB5476
   printf(Text);
#else
   bootPrint(Text);
#endif

   return Length;

}



//***************************************************************************
//	void	TimeMs
//
//	 returns system time in Milliseconds
//
//
//***************************************************************************
uint32 TimeMs(void)
{	
   unsigned long currentTime = 0;

    // I need to get the current system tick counter, then convert to ms.
    currentTime = tickGet();
          
    // Note that there is a risk of overflow here if ticks is greater than
    // about 4,300,000 because it will exceed a 32 bit number.  ticks will
    // roll over at some point anyway, so it doesn't really matter when.
    currentTime *= 1000;
    currentTime /= 100;
    
    return currentTime;

#if 0
   uint32	date, time, ticks;
	byte  	Second, Minute;
	uint32	ms;

	tm_get(&date, &time, &ticks);
	Second = (byte)(time & 0xFF);
	Minute = (byte)((time & 0xFF00) >> 8);
	ms = (10 * ticks) + (Second * 1000) + (Minute * 1000 * 60);
	return (ms);
#endif

} 
